###########################################################################
#                                                                         #
# trivia.tcl v0.3                              (c) 1997 by Tobias Begalke #
#                          email: tsbegalk@cip.informatik.uni-erlangen.de #
#                                                                         #
# An irc trivia game. For use with eggdrop bots v1.0 and newer.           #
#                                                                         #
# the users on a channel can start a new game by typing "!trivia [number] #
# [number] is optional! without number, the number of points needed to    #
#          win a match is $to_win (see below)                             #
# options include: "pf" (or your command, simply change "command" below)  #
#                   followed by                                           #
#   help         : show commands and options                              #
#   join team 1  : by typing this command, the user joins team 1          #
#   join team 2  : by typing this command, the user joins team 2          #
#   show teams   : shows who's on which team                              #
#   team <name>  : sets name of the team the user is on to <name>         #
#   show comment : displays the current question to the channel           #
#   show points  : displays the points of each team                       #
#   quit         : abort a running game without a result                  #
#                                                                         #
#-------------------------------------------------------------------------#
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the Free Software           #
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             #
#                                                                         #
###########################################################################


###########################################################################
# some global variables                                                   #
###########################################################################

set command pf           ;# command which prededes everything concerning the 
                         ;# game; change it to whatever you prefer.
set to_win 3             ;# number of points needed to win a match
set game_in_process 0    ;# indicates if there's a running game
set team_1 {}            ;# list of team-1-members
set team_2 {}            ;# list of team-2-members
set teamname_1 {Team 1}  ;# name of team 1
set teamname_2 {Team 2}  ;# name of team 2
set points_1 0           ;# points for team 1
set points_2 0           ;# points for team 2
set q_timer 0            ;# timerID for the question
set q_index -1           ;# index for selection of a question
set team_c 0             ;# team nr. of the team who found an answer
set found 0              ;# indicates if an answer was already found by a user
set nick_c nobody        ;# the nick of the one who found the answer
set intervall_s 50       ;# time from question to result (seconds)
set intervall_q 50       ;# time from result to next question (seconds)
set earlier_q {}         ;# list of earlier questions
set amount_q 0           ;# amount of questions already asked
set author "Comments, t-shirts, exotic food and new questions go to Eddiee (tsbegalk@cip.informatik.uni-erlangen.de)."

source scripts/questions1.tcl ;# import questions and answers, change at will

###########################################################################
# Some aliases for output                                                 #
###########################################################################

proc putchan {chan what} { putserv "privmsg $chan :$what" }
proc putmsg {nick msg} { putserv "privmsg $nick :$msg" }
proc putnot {nick msg} { puthelp "NOTICE $nick :$msg" }

###########################################################################
# Set a new teamname                                                      #
###########################################################################

proc set_teamname {nick name} {
    global team_1 team_2 teamname_1 teamname_2

    if {[string trim $name] == ""} { return 0 } ;# no teamname given

    if {[lsearch $team_1 $nick] != -1} {        ;# new name for team 1
	set teamname_1 [string trim $name]
    } elseif {[lsearch $team_2 $nick] != -1} {  ;# new name for team 2
	set teamname_2 [string trim $name]
    }
    return 0
}
###########################################################################
# Join team 1                                                             #
###########################################################################

proc join_team_1 {nick} {
    global team_1 team_2 teamname_1 teamname_2

    set i [lsearch $team_2 $nick]  ;# check if nick is on team 2
    if { $i != -1 } {              ;# remove it from this list, if yes
	set team_2 [lreplace $team_2 $i $i]
    }

    set i [lsearch $team_1 $nick]  ;# check if nick is already on team 1
    if { $i == -1 } {              ;# add it to list if not
	set team_1 [concat $team_1 $nick]
    }

    putmsg $nick "Joined Team 1, $teamname_1"

    return 0
}

###########################################################################
# Join team 2                                                             #
###########################################################################

proc join_team_2 {nick} {
    global team_1 team_2 teamname_1 teamname_2

    set i [lsearch $team_1 $nick]  ;# check if nick is on team 1
    if { $i != -1 } {              ;# remove it from this list, if yes
	set team_1 [lreplace $team_1 $i $i]
    }

    set i [lsearch $team_2 $nick]  ;# check if nick is already on team 2
    if { $i == -1 } {              ;# add it to list if not
	set team_2 [ concat $team_2 $nick ]
    }

    putmsg $nick "Joined Team 2, $teamname_2"

    return 0
}

###########################################################################
# Show team members and persons not on any team                           #
###########################################################################

proc show_teams {chan} {
    global team_1 team_2 teamname_1 teamname_2 botnick
    set users [chanlist $chan]        ;# put everyone on channel on userlist
    
    set i [lsearch $users $botnick]   ;# remove bots nick from userlist
    set users [lreplace $users $i $i]

    foreach value $team_1 {           ;# remove non-channel-members from teams
	if {[onchan $value $chan] != 1} {
	    set i [lsearch $team_1 $value]
	    set team_1 [lreplace $team_1 $i $i]
	}
    } 
    foreach value $team_2 {
	if {[onchan $value $chan] != 1} {
	    set i [lsearch $team_2 $value]
	    set team_2 [lreplace $team_2 $i $i]
	}
    }

    foreach value $team_1 {           ;# delete team-members from userlist
	set i [lsearch $users $value]
	set users [lreplace $users $i $i]
    }
    foreach value $team_2 {
	set i [lsearch $users $value]
	set users [lreplace $users $i $i]
    }

    putchan $chan " On no team :  $users"
    putchan $chan " On $teamname_1 :  $team_1"
    putchan $chan " On $teamname_2 :  $team_2"

    return 0
}

###########################################################################
# The actual main procedure                                               #
###########################################################################

proc game {nick uhost hand chan args} {
    global game_in_process team_1 team_2 q_index question answer
    global team_c found points_1 points_2 nick_c command
    global q_timer botnick teamname_1 teamname_2 amount_q

    set rest [string tolower $args]
    set rest [string trim $rest \{\}\ ]

    if { [string match {help} $rest] == 1 } {
	putmsg $nick " Available commands for this game are $command followed by: "
	putmsg $nick " help  shows this help"
	putmsg $nick " join team 1  lets you join team 1"
	putmsg $nick " join team 2  see above"
	putmsg $nick " show teams  shows, who's on a team currently"
	putmsg $nick " team <new name>  changes your team's name to <new name>"
	putmsg $nick " show points  displays the points of each team"
	putmsg $nick " show comment  redisplays the current question"
	putmsg $nick " quit  ends the current game "
    } elseif { [string match {show teams} $rest] == 1 } {
	show_teams $chan 
    } elseif { [string match {join team 1} $rest] == 1 } {
	join_team_1 $nick
    } elseif { [string match {join team 2} $rest] == 1 } {
	join_team_2 $nick
    } elseif { [string match {team?*} $rest] == 1 } {
	set name [string trim $args \{\}\ ]
	set name [lrange $name 1 end]
	set_teamname $nick $name
    } elseif { [string match {show comment} $rest] == 1 } {
	if { $q_index < 0 } { return 0 }
	putchan $chan " The current question is: $question($q_index)"
    } elseif { [string match {quit} $rest] == 1 } {
	killutimer $q_timer
	set game_in_process 0
	set points_1 0
	set points_2 0
	set found 0
	set team_1 {}
	set team_2 {}
	set team_c 0
	bind pub - $command no_game_yet
	putchan $chan " $nick cancelled the current game! "
	return 0
    } elseif { [string match {show points} $rest] == 1 } {
	putchan $chan " $teamname_1: $points_1, $teamname_2: $points_2 "
    } else {  ;# parse input to match the question and give away points
	if { $q_index < 0 } { return 0 }
	set ans_search [lrange $answer($q_index) 1 1]
	set ans_search [string trim $ans_search \{\}\ ]
	set ans_search [string tolower $ans_search]
	if { $found != 0 } {
	    return 0
	}
	if { [string match $ans_search $rest] == 1} {
	    if { [lsearch $team_1 $nick] >= 0 } {
		set found 1
		set nick_c $nick
		set team_c 1
	    } elseif { [lsearch $team_2 $nick] >= 0 } {
		set found 1
		set nick_c $nick
		set team_c 2
	    } else { return 0 }
    }   }
    return 0
}

###########################################################################
# this procedure starts the game and initializes everything               #
###########################################################################

proc startup {nick uhost hand chan args} {
    global game_in_process points_1 points_2 team_1 team_2 command
    global teamname_1 teamname_2 q_timer no_questions to_win
    global a_timer nick_c botnick team_c intervall_q earlier_q amount_q

    if { $game_in_process != 0 } { ;# do nothing if a game is already running
	putmsg $nick "Game in process! Join a team please, or type $command help for help."
	return 0
    }

    set game_in_process 1          ;# a new game is running
    set nick_c nobody
    bind pub - $command game
#    set earlier_q {}  ;# unmark these 2 lines, if you want the complete
#    set amount_q 0    ;# set of questions each game
    set team_1 {}                  ;# reset team lists
    set team_2 {}
    set team_c 0
    set teamname_1 {Team 1}        ;# set default teamnames
    set teamname_2 {Team 2}

    set arg [string trim $args \{\}\ ]  ;# get number of points needed to win 
    if { ($arg != {}) && ($arg <= 10) && ($arg > 0) } {
	set to_win $arg
    } elseif { ($arg != {}) && ($arg > 0) } {
	set to_win 10
    } else { set to_win 3 }
    set q_timer [utimer $intervall_q "ask $chan"] ;# set timer for the question

    putchan $chan " This is $botnick, your trivia bot           "
    putchan $chan " Starting a new game. $to_win points needed to win "
    putchan $chan " Players please join a team now by typing \"$command join team 1\" or \"$command join team 2\""
    putchan $chan " type \"$command help\" for help. "


    return 0
}

###########################################################################
# finds a question not yet asked in this round and returns its index      #
###########################################################################

proc find_index {max} {
    global earlier_q no_questions amount_q

    if { $amount_q >= $no_questions } { 
	set earlier_q {}
	set amount_q 0
    }

    set i [rand $max]

    while { [lsearch $earlier_q $i] != -1 } { 
	incr i
	set i [expr $i % $no_questions]
    }
    set earlier_q [concat $earlier_q $i]
    incr amount_q
    return $i
}
    

###########################################################################
# calculates and prints the question                                      #
###########################################################################

proc ask {chan} {
    global team_1 team_2 question no_questions q_index earlier_q
    global points_1 points_2 to_win q_timer team_c intervall_s amount_q

    set team_c 0
    set found 0                      ;# set found back to 0
    set i [find_index $no_questions] ;# generate index of question
    set q_index $i

    putchan $chan " The question is:  $question($i)"
    set q_timer [utimer $intervall_s "show $chan"]  ;# set timer for the result output
    return 0
}

###########################################################################
# shows the winning team and resets everything                            #
###########################################################################

proc win {team_no chan} {
    global team_1 team_2 points_1 points_2 nick_c author
    global teamname_1 teamname_2 found game_in_process command

    if { $team_no == 1 } { ;# check which team wins
	putchan $chan " $teamname_1 (Team 1) wins this match! "
    } else {
	putchan $chan " $teamname_2 (Team 2) wins this match! "
    }
    putchan $chan " The result is $teamname_1: $points_1, $teamname_2: $points_2 "
    putchan $chan " $author"
    bind pub - $command no_game_yet ;# reset all functions and vars
    set points_1 0
    set points_2 0
    set found 0
    set game_in_process 0
    set nick_c nobody

    return 0
}

###########################################################################
# prints the result of a round and terminates the game                    #
###########################################################################

proc show {chan} {
    global nick_c team_c points_1 points_2 answer q_timer intervall_q
    global q_index team_1 team_2 teamname_1 teamname_2 found to_win

    set p_1 Points
    set p_2 Points
    set i $q_index
    set q_index -1  ;# accept no more input

    set output [lrange $answer($i) 0 0]     ;# set output string
    set output [string trim $output \{\}\ ]

    putchan $chan " Here's the result of this round: "
    putchan $chan " The correct answer was: $output"
    if { $team_c == 1 } { ;# team 1 won the round
        incr points_1
	putchan $chan " Found by $nick_c -=$teamname_1=- "
    } elseif { $team_c == 2 } { ;# team 2 won the round
        incr points_2
	putchan $chan " Found by $nick_c -=$teamname_2=- "
    } else { putchan $chan " Found by $nick_c" }
    if { $points_1 == 1 } { set p_1 Point }
    if { $points_2 == 1 } { set p_2 Point }
    putchan $chan " $teamname_1 : $points_1 $p_1"
    putchan $chan " $teamname_2 : $points_2 $p_2"

    if { $points_1 >= $to_win } {      ;# display the final result if a team
	if { $points_1 > $points_2 } { ;# enough points
	    win 1 $chan
	    return 0
    }   }
    if { $points_2 >= $to_win } {
	if { $points_2 > $points_1 } {
	    win 2 $chan
	    return 0
    }   }

    set nick_c nobody  ;# reset the adequate vars
    set found 0
    set q_timer [utimer $intervall_q "ask $chan"] ;# set timer for next question

    return 0
}

proc no_game_yet {nick uhost hand chan args} {
    putmsg $nick {There's no game running! Start one by typing !trivia [1-10]}
    putmsg $nick "The number is optional. By default, 3 points are needed to win a match. To play a game with 6 points needed to win, type !trivia 6."
    return 0
}

###########################################################################
# if a user changes nick, this procedure updates the teamlist             #
###########################################################################

proc update_team {nick uhost handle chan newnick} {
    global team_1 team_2 nick_c
    
    if { [string match $nick_c $nick] == 1 } {
	set nick_c $newnick
    }

    if { [lsearch $nick $team_1] >= 0 } {
	set team_1 [concat $team_1 $newnick]
    } elseif { [lsearch $nick $team_2] >= 0 } {
	set team_2 [concat $team_2 $newnick]
    }
    return 0
}

bind pub - $command no_game_yet
bind pub - !trivia startup
bind nick - * update_team


